@@ -1,8 +1,8 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 """
 Wrapper for libsodium library
 
-Copyright (c) 2013-2014, Marsiske Stefan.
+Copyright (c) 2013-2023, Marsiske Stefan.
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification,
@@ -120,6 +120,8 @@ crypto_stream_KEYBYTES = sodium.crypto_stream_keybytes()
 crypto_stream_NONCEBYTES = sodium.crypto_stream_noncebytes()
 crypto_stream_chacha20_NONCEBYTES = sodium.crypto_stream_chacha20_noncebytes()
 crypto_stream_chacha20_KEYBYTES = sodium.crypto_stream_chacha20_keybytes()
+crypto_stream_chacha20_ietf_NONCEBYTES = sodium.crypto_stream_chacha20_ietf_noncebytes()
+crypto_stream_chacha20_ietf_KEYBYTES = sodium.crypto_stream_chacha20_ietf_keybytes()
 crypto_stream_xchacha20_NONCEBYTES = sodium.crypto_stream_xchacha20_noncebytes()
 crypto_stream_xchacha20_KEYBYTES = sodium.crypto_stream_xchacha20_keybytes()
 crypto_generichash_KEYBYTES_MAX = sodium.crypto_generichash_keybytes_max()
@@ -150,6 +152,10 @@ crypto_aead_chacha20poly1305_KEYBYTES = sodium.crypto_aead_chacha20poly1305_keyb
 crypto_aead_chacha20poly1305_NPUBBYTES = sodium.crypto_aead_chacha20poly1305_npubbytes()
 crypto_aead_chacha20poly1305_NONCEBYTES = crypto_aead_chacha20poly1305_NPUBBYTES
 crypto_aead_chacha20poly1305_ABYTES = sodium.crypto_aead_chacha20poly1305_abytes()
+crypto_kdf_BYTES_MIN = sodium.crypto_kdf_bytes_min()
+crypto_kdf_BYTES_MAX = sodium.crypto_kdf_bytes_max()
+crypto_kdf_CONTEXTBYTES = sodium.crypto_kdf_contextbytes()
+crypto_kdf_KEYBYTES = sodium.crypto_kdf_keybytes()
 
 if sodium_version_check(1, 0, 9):
     crypto_aead_chacha20poly1305_ietf_KEYBYTES = sodium.crypto_aead_chacha20poly1305_ietf_keybytes()
@@ -243,6 +249,7 @@ if sodium_version_check(1, 0, 15):
     crypto_secretstream_xchacha20poly1305_ABYTES = sodium.crypto_secretstream_xchacha20poly1305_abytes()
     crypto_secretstream_xchacha20poly1305_HEADERBYTES = sodium.crypto_secretstream_xchacha20poly1305_headerbytes()
     crypto_secretstream_xchacha20poly1305_KEYBYTES = sodium.crypto_secretstream_xchacha20poly1305_keybytes()
+    sodium.crypto_secretstream_xchacha20poly1305_messagebytes_max.restype = ctypes.c_size_t
     crypto_secretstream_xchacha20poly1305_MESSAGEBYTES_MAX = sodium.crypto_secretstream_xchacha20poly1305_messagebytes_max()
     crypto_secretstream_xchacha20poly1305_TAG_MESSAGE = sodium.crypto_secretstream_xchacha20poly1305_tag_message()
     crypto_secretstream_xchacha20poly1305_TAG_PUSH = sodium.crypto_secretstream_xchacha20poly1305_tag_push()
@@ -255,9 +262,37 @@ if sodium_version_check(1, 0, 18):
     crypto_core_ristretto255_SCALARBYTES = sodium.crypto_core_ristretto255_scalarbytes()
     crypto_core_ristretto255_NONREDUCEDSCALARBYTES = sodium.crypto_core_ristretto255_nonreducedscalarbytes()
     crypto_auth_hmacsha256_BYTES = sodium.crypto_auth_hmacsha256_bytes()
+    crypto_auth_hmacsha256_KEYBYTES = sodium.crypto_auth_hmacsha256_keybytes()
     crypto_hash_sha256_STATEBYTES = sodium.crypto_hash_sha256_statebytes()
     crypto_auth_hmacsha512_BYTES = sodium.crypto_auth_hmacsha512_bytes()
+    crypto_auth_hmacsha512_KEYBYTES = sodium.crypto_auth_hmacsha512_keybytes()
     crypto_hash_sha512_STATEBYTES = sodium.crypto_hash_sha512_statebytes()
+    crypto_auth_hmacsha512256_BYTES = sodium.crypto_auth_hmacsha512256_bytes()
+    crypto_auth_hmacsha512256_KEYBYTES = sodium.crypto_auth_hmacsha512256_keybytes()
+
+if sodium_version_check(1, 0, 19):
+    crypto_kdf_hkdf_sha256_KEYBYTES = sodium.crypto_kdf_hkdf_sha256_keybytes()
+    crypto_kdf_hkdf_sha256_BYTES_MIN = sodium.crypto_kdf_hkdf_sha256_bytes_min()
+    crypto_kdf_hkdf_sha256_BYTES_MAX = sodium.crypto_kdf_hkdf_sha256_bytes_max()
+    crypto_kdf_hkdf_sha256_STATEBYTES = sodium.crypto_kdf_hkdf_sha256_statebytes()
+    crypto_kdf_hkdf_sha512_KEYBYTES = sodium.crypto_kdf_hkdf_sha512_keybytes()
+    crypto_kdf_hkdf_sha512_BYTES_MIN = sodium.crypto_kdf_hkdf_sha512_bytes_min()
+    crypto_kdf_hkdf_sha512_BYTES_MAX = sodium.crypto_kdf_hkdf_sha512_bytes_max()
+    crypto_aead_aegis128l_KEYBYTES = sodium.crypto_aead_aegis128l_keybytes()
+    crypto_aead_aegis128l_NPUBBYTES = sodium.crypto_aead_aegis128l_npubbytes()
+    crypto_aead_aegis128l_NONCEBYTES = crypto_aead_aegis128l_NPUBBYTES
+    crypto_aead_aegis128l_ABYTES = sodium.crypto_aead_aegis128l_abytes()
+    sodium.crypto_aead_aegis128l_messagebytes_max.restype = ctypes.c_size_t
+    crypto_aead_aegis128l_MESSAGEBYTES_MAX = sodium.crypto_aead_aegis128l_messagebytes_max()
+    crypto_aead_aegis256_KEYBYTES = sodium.crypto_aead_aegis256_keybytes()
+    crypto_aead_aegis256_NPUBBYTES = sodium.crypto_aead_aegis256_npubbytes()
+    crypto_aead_aegis256_NONCEBYTES = crypto_aead_aegis256_NPUBBYTES
+    crypto_aead_aegis256_ABYTES = sodium.crypto_aead_aegis256_abytes()
+    sodium.crypto_aead_aegis256_messagebytes_max.restype = ctypes.c_size_t
+    crypto_aead_aegis256_MESSAGEBYTES_MAX = sodium.crypto_aead_aegis256_messagebytes_max()
+
+if sodium_version_check(1, 0, 20):
+    crypto_kdf_hkdf_sha512_STATEBYTES = sodium.crypto_kdf_hkdf_sha512_statebytes()
 
 sodium_init = sodium.sodium_init
 
@@ -339,6 +374,33 @@ def crypto_stream_chacha20_xor_ic(message, nonce, initial_counter, key):
 
     return c.raw
 
+# crypto_stream_chacha20_ietf_xor(unsigned char *c, const unsigned char *m, unsigned long long mlen, const unsigned char *n, const unsigned char *k)
+def crypto_stream_chacha20_ietf_xor(message, nonce, key):
+    if len(nonce) != crypto_stream_chacha20_ietf_NONCEBYTES: raise ValueError("truncated nonce")
+    if len(key) != crypto_stream_chacha20_ietf_KEYBYTES: raise ValueError("truncated key")
+
+    mlen = ctypes.c_longlong(len(message))
+
+    c = ctypes.create_string_buffer(len(message))
+
+    __check(sodium.crypto_stream_chacha20_ietf_xor(c, message, mlen, nonce, key))
+
+    return c.raw
+
+# crypto_stream_chacha20_ietf_xor_ic(unsigned char *c, const unsigned char *m, unsigned long long mlen, const unsigned char *n, uint64_t ic, const unsigned char *k)
+def crypto_stream_chacha20_ietf_xor_ic(message, nonce, initial_counter, key):
+    if len(nonce) != crypto_stream_chacha20_ietf_NONCEBYTES: raise ValueError("truncated nonce")
+    if len(key) != crypto_stream_chacha20_ietf_KEYBYTES: raise ValueError("truncated key")
+
+    mlen = ctypes.c_longlong(len(message))
+    ic = ctypes.c_uint64(initial_counter)
+
+    c = ctypes.create_string_buffer(len(message))
+
+    __check(sodium.crypto_stream_chacha20_ietf_xor_ic(c, message, mlen, nonce, ic, key))
+
+    return c.raw
+
 # crypto_stream_xchacha20_xor(unsigned char *c, const unsigned char *m, unsigned long long mlen, const unsigned char *n, const unsigned char *k)
 def crypto_stream_xchacha20_xor(message, nonce, key):
     if len(nonce) != crypto_stream_xchacha20_NONCEBYTES: raise ValueError("truncated nonce")
@@ -374,7 +436,7 @@ def crypto_aead_chacha20poly1305_encrypt(message, ad, nonce, key):
     mlen = ctypes.c_ulonglong(len(message))
     adlen = ctypes.c_ulonglong(len(ad)) if ad is not None else ctypes.c_ulonglong(0)
 
-    c = ctypes.create_string_buffer(mlen.value + 16)
+    c = ctypes.create_string_buffer(mlen.value + crypto_aead_chacha20poly1305_ABYTES)
     clen = ctypes.c_ulonglong(0)
 
     __check(sodium.crypto_aead_chacha20poly1305_encrypt(c, ctypes.byref(clen), message, mlen, ad, adlen, None, nonce, key))
@@ -386,7 +448,7 @@ def crypto_aead_chacha20poly1305_decrypt(ciphertext, ad, nonce, key):
     if len(nonce) != crypto_aead_chacha20poly1305_NONCEBYTES: raise ValueError("truncated nonce")
     if len(key) != crypto_aead_chacha20poly1305_KEYBYTES: raise ValueError("truncated key")
 
-    m = ctypes.create_string_buffer(len(ciphertext) - 16)
+    m = ctypes.create_string_buffer(len(ciphertext) - crypto_aead_chacha20poly1305_ABYTES)
     mlen = ctypes.c_ulonglong(0)
     clen = ctypes.c_ulonglong(len(ciphertext))
     adlen = ctypes.c_ulonglong(len(ad)) if ad is not None else ctypes.c_ulonglong(0)
@@ -424,6 +486,126 @@ def crypto_aead_chacha20poly1305_decrypt_detached(ciphertext, mac, ad, nonce, ke
     __check(sodium.crypto_aead_chacha20poly1305_decrypt_detached(m, None, ciphertext, clen, mac, ad, adlen, nonce, key))
     return m.raw
 
+# crypto_aead_aegis128l_encrypt(unsigned char *c, unsigned long long *clen, const unsigned char *m, unsigned long long mlen, const unsigned char *ad, unsigned long long adlen, const unsigned char *nsec, const unsigned char *npub, const unsigned char *k);
+@sodium_version(1, 0, 19)
+def crypto_aead_aegis128l_encrypt(message, ad, nonce, key):
+    if len(nonce) != crypto_aead_aegis128l_NONCEBYTES: raise ValueError("truncated nonce")
+    if len(key) != crypto_aead_aegis128l_KEYBYTES: raise ValueError("truncated key")
+
+    mlen = ctypes.c_ulonglong(len(message))
+    adlen = ctypes.c_ulonglong(len(ad)) if ad is not None else ctypes.c_ulonglong(0)
+
+    c = ctypes.create_string_buffer(mlen.value + crypto_aead_aegis128l_ABYTES)
+    clen = ctypes.c_ulonglong(0)
+
+    __check(sodium.crypto_aead_aegis128l_encrypt(c, ctypes.byref(clen), message, mlen, ad, adlen, None, nonce, key))
+    return c.raw
+
+
+# crypto_aead_aegis128l_decrypt(unsigned char *m, unsigned long long *mlen, unsigned char *nsec, const unsigned char *c, unsigned long long clen, const unsigned char *ad, unsigned long long adlen, const unsigned char *npub, const unsigned char *k)
+@sodium_version(1, 0, 19)
+def crypto_aead_aegis128l_decrypt(ciphertext, ad, nonce, key):
+    if len(nonce) != crypto_aead_aegis128l_NONCEBYTES: raise ValueError("truncated nonce")
+    if len(key) != crypto_aead_aegis128l_KEYBYTES: raise ValueError("truncated key")
+
+    m = ctypes.create_string_buffer(len(ciphertext) - crypto_aead_aegis128l_ABYTES)
+    mlen = ctypes.c_ulonglong(0)
+    clen = ctypes.c_ulonglong(len(ciphertext))
+    adlen = ctypes.c_ulonglong(len(ad)) if ad is not None else ctypes.c_ulonglong(0)
+    __check(sodium.crypto_aead_aegis128l_decrypt(m, ctypes.byref(mlen), None, ciphertext, clen, ad, adlen, nonce, key))
+    return m.raw
+
+# crypto_aead_aegis128l_encrypt_detached(unsigned char *c, unsigned char *mac, unsigned long long *maclen_p, const unsigned char *m, unsigned long long mlen, const unsigned char *ad, unsigned long long adlen, const unsigned char *nsec, const unsigned char *npub, const unsigned char *k)
+@sodium_version(1, 0, 19)
+def crypto_aead_aegis128l_encrypt_detached(message, ad, nonce, key):
+    """ Return ciphertext, mac tag """
+    if len(nonce) != crypto_aead_aegis128l_NONCEBYTES: raise ValueError("truncated nonce")
+    if len(key) != crypto_aead_aegis128l_KEYBYTES: raise ValueError("truncated key")
+
+    mlen = ctypes.c_ulonglong(len(message))
+    adlen = ctypes.c_ulonglong(len(ad)) if ad is not None else ctypes.c_ulonglong(0)
+    c = ctypes.create_string_buffer(mlen.value)
+    maclen_p = ctypes.c_ulonglong(crypto_aead_aegis128l_ABYTES)
+    mac = ctypes.create_string_buffer(maclen_p.value)
+
+    __check(sodium.crypto_aead_aegis128l_encrypt_detached(c, mac, ctypes.byref(maclen_p), message, mlen, ad, adlen, None, nonce, key))
+    return c.raw, mac.raw
+
+# crypto_aead_aegis128l_decrypt_detached(unsigned char *m, unsigned char *nsec, const unsigned char *c, unsigned long long clen, const unsigned char *mac, const unsigned char *ad, unsigned long long adlen, const unsigned char *npub, const unsigned char *k)
+@sodium_version(1, 0, 19)
+def crypto_aead_aegis128l_decrypt_detached(ciphertext, mac, ad, nonce, key):
+    """ Return message if successful or -1 (ValueError) if not successful"""
+    if len(nonce) != crypto_aead_aegis128l_NONCEBYTES: raise ValueError("truncated nonce")
+    if len(key) != crypto_aead_aegis128l_KEYBYTES: raise ValueError("truncated key")
+    if len(mac) != crypto_aead_aegis128l_ABYTES:
+        raise ValueError("mac length != %i" % crypto_aead_aegis128l_ABYTES)
+
+    clen = ctypes.c_ulonglong(len(ciphertext))
+    m = ctypes.create_string_buffer(clen.value)
+    adlen = ctypes.c_ulonglong(len(ad)) if ad is not None else ctypes.c_ulonglong(0)
+    __check(sodium.crypto_aead_aegis128l_decrypt_detached(m, None, ciphertext, clen, mac, ad, adlen, nonce, key))
+    return m.raw
+
+# crypto_aead_aegis256_encrypt(unsigned char *c, unsigned long long *clen, const unsigned char *m, unsigned long long mlen, const unsigned char *ad, unsigned long long adlen, const unsigned char *nsec, const unsigned char *npub, const unsigned char *k);
+@sodium_version(1, 0, 19)
+def crypto_aead_aegis256_encrypt(message, ad, nonce, key):
+    if len(nonce) != crypto_aead_aegis256_NONCEBYTES: raise ValueError("truncated nonce")
+    if len(key) != crypto_aead_aegis256_KEYBYTES: raise ValueError("truncated key")
+
+    mlen = ctypes.c_ulonglong(len(message))
+    adlen = ctypes.c_ulonglong(len(ad)) if ad is not None else ctypes.c_ulonglong(0)
+
+    c = ctypes.create_string_buffer(mlen.value + crypto_aead_aegis256_ABYTES)
+    clen = ctypes.c_ulonglong(0)
+
+    __check(sodium.crypto_aead_aegis256_encrypt(c, ctypes.byref(clen), message, mlen, ad, adlen, None, nonce, key))
+    return c.raw
+
+
+# crypto_aead_aegis256_decrypt(unsigned char *m, unsigned long long *mlen, unsigned char *nsec, const unsigned char *c, unsigned long long clen, const unsigned char *ad, unsigned long long adlen, const unsigned char *npub, const unsigned char *k)
+@sodium_version(1, 0, 19)
+def crypto_aead_aegis256_decrypt(ciphertext, ad, nonce, key):
+    if len(nonce) != crypto_aead_aegis256_NONCEBYTES: raise ValueError("truncated nonce")
+    if len(key) != crypto_aead_aegis256_KEYBYTES: raise ValueError("truncated key")
+
+    m = ctypes.create_string_buffer(len(ciphertext) - crypto_aead_aegis256_ABYTES)
+    mlen = ctypes.c_ulonglong(0)
+    clen = ctypes.c_ulonglong(len(ciphertext))
+    adlen = ctypes.c_ulonglong(len(ad)) if ad is not None else ctypes.c_ulonglong(0)
+    __check(sodium.crypto_aead_aegis256_decrypt(m, ctypes.byref(mlen), None, ciphertext, clen, ad, adlen, nonce, key))
+    return m.raw
+
+# crypto_aead_aegis256_encrypt_detached(unsigned char *c, unsigned char *mac, unsigned long long *maclen_p, const unsigned char *m, unsigned long long mlen, const unsigned char *ad, unsigned long long adlen, const unsigned char *nsec, const unsigned char *npub, const unsigned char *k)
+@sodium_version(1, 0, 19)
+def crypto_aead_aegis256_encrypt_detached(message, ad, nonce, key):
+    """ Return ciphertext, mac tag """
+    if len(nonce) != crypto_aead_aegis256_NONCEBYTES: raise ValueError("truncated nonce")
+    if len(key) != crypto_aead_aegis256_KEYBYTES: raise ValueError("truncated key")
+
+    mlen = ctypes.c_ulonglong(len(message))
+    adlen = ctypes.c_ulonglong(len(ad)) if ad is not None else ctypes.c_ulonglong(0)
+    c = ctypes.create_string_buffer(mlen.value)
+    maclen_p = ctypes.c_ulonglong(crypto_aead_aegis256_ABYTES)
+    mac = ctypes.create_string_buffer(maclen_p.value)
+
+    __check(sodium.crypto_aead_aegis256_encrypt_detached(c, mac, ctypes.byref(maclen_p), message, mlen, ad, adlen, None, nonce, key))
+    return c.raw, mac.raw
+
+# crypto_aead_aegis256_decrypt_detached(unsigned char *m, unsigned char *nsec, const unsigned char *c, unsigned long long clen, const unsigned char *mac, const unsigned char *ad, unsigned long long adlen, const unsigned char *npub, const unsigned char *k)
+@sodium_version(1, 0, 19)
+def crypto_aead_aegis256_decrypt_detached(ciphertext, mac, ad, nonce, key):
+    """ Return message if successful or -1 (ValueError) if not successful"""
+    if len(nonce) != crypto_aead_aegis256_NONCEBYTES: raise ValueError("truncated nonce")
+    if len(key) != crypto_aead_aegis256_KEYBYTES: raise ValueError("truncated key")
+    if len(mac) != crypto_aead_aegis256_ABYTES:
+        raise ValueError("mac length != %i" % crypto_aead_aegis256_ABYTES)
+
+    clen = ctypes.c_ulonglong(len(ciphertext))
+    m = ctypes.create_string_buffer(clen.value)
+    adlen = ctypes.c_ulonglong(len(ad)) if ad is not None else ctypes.c_ulonglong(0)
+    __check(sodium.crypto_aead_aegis256_decrypt_detached(m, None, ciphertext, clen, mac, ad, adlen, nonce, key))
+    return m.raw
+
 # crypto_aead_chacha20poly1305_ietf_encrypt(unsigned char *c, unsigned long long *clen_p, const unsigned char *m, unsigned long long mlen, const unsigned char *ad, unsigned long long adlen, const unsigned char *nsec, const unsigned char *npub, const unsigned char *k)
 @sodium_version(1, 0, 4)
 def crypto_aead_chacha20poly1305_ietf_encrypt(message, ad, nonce, key):
@@ -432,7 +614,7 @@ def crypto_aead_chacha20poly1305_ietf_encrypt(message, ad, nonce, key):
 
     mlen = ctypes.c_ulonglong(len(message))
     adlen = ctypes.c_ulonglong(len(ad)) if ad is not None else ctypes.c_ulonglong(0)
-    c = ctypes.create_string_buffer(mlen.value + 16)
+    c = ctypes.create_string_buffer(mlen.value + crypto_aead_chacha20poly1305_ietf_ABYTES)
     clen = ctypes.c_ulonglong(0)
 
     __check(sodium.crypto_aead_chacha20poly1305_ietf_encrypt(c, ctypes.byref(clen), message, mlen, ad, adlen, None, nonce, key))
@@ -444,7 +626,7 @@ def crypto_aead_chacha20poly1305_ietf_decrypt(ciphertext, ad, nonce, key):
     if len(nonce) != crypto_aead_chacha20poly1305_ietf_NONCEBYTES: raise ValueError("truncated nonce")
     if len(key) != crypto_aead_chacha20poly1305_ietf_KEYBYTES: raise ValueError("truncated key")
 
-    m = ctypes.create_string_buffer(len(ciphertext) - 16)
+    m = ctypes.create_string_buffer(len(ciphertext) - crypto_aead_chacha20poly1305_ietf_ABYTES)
     mlen = ctypes.c_ulonglong(0)
     clen = ctypes.c_ulonglong(len(ciphertext))
     adlen = ctypes.c_ulonglong(len(ad)) if ad is not None else ctypes.c_ulonglong(0)
@@ -492,7 +674,7 @@ def crypto_aead_xchacha20poly1305_ietf_encrypt(message, ad, nonce, key):
     if len(key) != crypto_aead_xchacha20poly1305_ietf_KEYBYTES: raise ValueError("truncated key")
     mlen = ctypes.c_ulonglong(len(message))
     adlen = ctypes.c_ulonglong(len(ad)) if ad is not None else ctypes.c_ulonglong(0)
-    c = ctypes.create_string_buffer(mlen.value + 16)
+    c = ctypes.create_string_buffer(mlen.value + crypto_aead_xchacha20poly1305_ietf_ABYTES)
     clen = ctypes.c_ulonglong(0)
 
     __check(sodium.crypto_aead_xchacha20poly1305_ietf_encrypt(c, ctypes.byref(clen),
@@ -511,7 +693,7 @@ def crypto_aead_xchacha20poly1305_ietf_decrypt(ciphertext, ad, nonce, key):
     if len(nonce) != crypto_aead_xchacha20poly1305_ietf_NPUBBYTES: raise ValueError("truncated nonce")
     if len(key) != crypto_aead_xchacha20poly1305_ietf_KEYBYTES: raise ValueError("truncated key")
 
-    m = ctypes.create_string_buffer(len(ciphertext) - 16)
+    m = ctypes.create_string_buffer(len(ciphertext) - crypto_aead_xchacha20poly1305_ietf_ABYTES)
     mlen = ctypes.c_ulonglong(0)
     clen = ctypes.c_ulonglong(len(ciphertext))
     adlen = ctypes.c_ulonglong(len(ad)) if ad is not None else ctypes.c_ulonglong(0)
@@ -542,6 +724,145 @@ def crypto_auth_verify(h, m, k):
         raise ValueError("invalid tag")
     __check(sodium.crypto_auth_verify(h, m, ctypes.c_ulonglong(len(m)), k))
 
+
+# void crypto_auth_hmacsha256_keygen(unsigned char k[crypto_auth_hmacsha256_KEYBYTES]);
+def crypto_auth_hmacsha256_keygen():
+    k = ctypes.create_string_buffer(crypto_auth_hmacsha256_KEYBYTES)
+    sodium.crypto_auth_hmacsha256_keygen(k)
+    return k.raw
+
+
+# int crypto_auth_hmacsha256(unsigned char *out,
+#                            const unsigned char *in,
+#                            unsigned long long inlen,
+#                            const unsigned char *k);
+def crypto_auth_hmacsha256(m, k):
+    if not len(k) == crypto_auth_hmacsha256_KEYBYTES:
+        raise ValueError("invalid parameters")
+    buf = ctypes.create_string_buffer(crypto_auth_hmacsha256_BYTES)
+    msg = ctypes.create_string_buffer(m)
+    key = ctypes.create_string_buffer(k)
+    __check(
+        sodium.crypto_auth_hmacsha256(
+            ctypes.byref(buf),
+            ctypes.byref(msg),
+            ctypes.c_ulonglong(len(m)),
+            ctypes.byref(key),
+        )
+    )
+    return buf.raw
+
+
+# int crypto_auth_hmacsha256_verify(const unsigned char *h,
+#                                   const unsigned char *in,
+#                                   unsigned long long inlen,
+#                                   const unsigned char *k);
+def crypto_auth_hmacsha256_verify(h, m, k):
+    if not len(k) == crypto_auth_hmacsha256_KEYBYTES:
+        raise ValueError("invalid parameters")
+    hmac = ctypes.create_string_buffer(h)
+    msg = ctypes.create_string_buffer(m)
+    mlen = ctypes.c_ulonglong(len(m))
+    key = ctypes.create_string_buffer(k)
+    __check(
+        sodium.crypto_auth_hmacsha256_verify(
+            ctypes.byref(hmac), ctypes.byref(msg), mlen, ctypes.byref(key)
+        )
+    )
+
+
+# void crypto_auth_hmacsha512_keygen(unsigned char k[crypto_auth_hmacsha512_KEYBYTES]);
+def crypto_auth_hmacsha512_keygen():
+    k = ctypes.create_string_buffer(crypto_auth_hmacsha512_KEYBYTES)
+    sodium.crypto_auth_hmacsha512_keygen(k)
+    return k.raw
+
+
+# int crypto_auth_hmacsha512(unsigned char *out,
+#                            const unsigned char *in,
+#                            unsigned long long inlen,
+#                            const unsigned char *k);
+def crypto_auth_hmacsha512(m, k):
+    if not len(k) == crypto_auth_hmacsha512_KEYBYTES:
+        raise ValueError("invalid parameters")
+    buf = ctypes.create_string_buffer(crypto_auth_hmacsha512_BYTES)
+    msg = ctypes.create_string_buffer(m)
+    key = ctypes.create_string_buffer(k)
+    __check(
+        sodium.crypto_auth_hmacsha512(
+            ctypes.byref(buf),
+            ctypes.byref(msg),
+            ctypes.c_ulonglong(len(m)),
+            ctypes.byref(key),
+        )
+    )
+    return buf.raw
+
+
+# int crypto_auth_hmacsha512_verify(const unsigned char *h,
+#                                   const unsigned char *in,
+#                                   unsigned long long inlen,
+#                                   const unsigned char *k);
+def crypto_auth_hmacsha512_verify(h, m, k):
+    if not len(k) == crypto_auth_hmacsha512_KEYBYTES:
+        raise ValueError("invalid parameters")
+    hmac = ctypes.create_string_buffer(h)
+    msg = ctypes.create_string_buffer(m)
+    mlen = ctypes.c_ulonglong(len(m))
+    key = ctypes.create_string_buffer(k)
+    __check(
+        sodium.crypto_auth_hmacsha512_verify(
+            ctypes.byref(hmac), ctypes.byref(msg), mlen, ctypes.byref(key)
+        )
+    )
+
+
+# void crypto_auth_hmacsha512256_keygen(unsigned char k[crypto_auth_hmacsha512256_KEYBYTES]);
+def crypto_auth_hmacsha512256_keygen():
+    k = ctypes.create_string_buffer(crypto_auth_hmacsha512256_KEYBYTES)
+    sodium.crypto_auth_hmacsha512256_keygen(k)
+    return k.raw
+
+
+# int crypto_auth_hmacsha512256(unsigned char *out,
+#                            const unsigned char *in,
+#                            unsigned long long inlen,
+#                            const unsigned char *k);
+def crypto_auth_hmacsha512256(m, k):
+    if not len(k) == crypto_auth_hmacsha512256_KEYBYTES:
+        raise ValueError("invalid parameters")
+    buf = ctypes.create_string_buffer(crypto_auth_hmacsha512256_BYTES)
+    msg = ctypes.create_string_buffer(m)
+    key = ctypes.create_string_buffer(k)
+    __check(
+        sodium.crypto_auth_hmacsha512256(
+            ctypes.byref(buf),
+            ctypes.byref(msg),
+            ctypes.c_ulonglong(len(m)),
+            ctypes.byref(key),
+        )
+    )
+    return buf.raw
+
+
+# int crypto_auth_hmacsha512256_verify(const unsigned char *h,
+#                                   const unsigned char *in,
+#                                   unsigned long long inlen,
+#                                   const unsigned char *k);
+def crypto_auth_hmacsha512256_verify(h, m, k):
+    if not len(k) == crypto_auth_hmacsha512256_KEYBYTES:
+        raise ValueError("invalid parameters")
+    hmac = ctypes.create_string_buffer(h)
+    msg = ctypes.create_string_buffer(m)
+    mlen = ctypes.c_ulonglong(len(m))
+    key = ctypes.create_string_buffer(k)
+    __check(
+        sodium.crypto_auth_hmacsha512256_verify(
+            ctypes.byref(hmac), ctypes.byref(msg), mlen, ctypes.byref(key)
+        )
+    )
+
+
 # crypto_generichash(unsigned char *out, size_t outlen, const unsigned char *in, unsigned long long inlen, const unsigned char *key, size_t keylen)
 @encode_strings
 def crypto_generichash(m, k=b'', outlen=crypto_generichash_BYTES):
@@ -710,10 +1031,10 @@ def crypto_box_seal_open(c, pk, sk):
     return msg.raw
 
 # int crypto_secretbox_detached(unsigned char *c, unsigned char *mac,
-#                              	const unsigned char *m,
-#                              	unsigned long long mlen,
-#                              	const unsigned char *n,
-#                              	const unsigned char *k);
+#                               const unsigned char *m,
+#                               unsigned long long mlen,
+#                               const unsigned char *n,
+#                               const unsigned char *k);
 
 def crypto_secretbox_detached(msg, nonce, k):
     if None in (msg, nonce, k): raise ValueError("invalid parameters")
@@ -1173,6 +1494,131 @@ def crypto_hash_sha512_final(state):
     __check(sodium.crypto_hash_sha512_final(state, out))
     return out.raw
 
+# int crypto_kdf_derive_from_key(unsigned char *subkey, size_t subkey_len,
+#                                uint64_t subkey_id,
+#                                const char ctx[crypto_kdf_CONTEXTBYTES],
+#                                const unsigned char key[crypto_kdf_KEYBYTES])
+def crypto_kdf_derive_from_key(subkey_len, subkey_id, ctx, key):
+    if len(ctx) != crypto_kdf_CONTEXTBYTES: raise ValueError("invalid context")
+    if len(key) != crypto_kdf_KEYBYTES: raise ValueError("invalid key")
+    if not (crypto_kdf_BYTES_MIN <= subkey_len <= crypto_kdf_BYTES_MAX): raise ValueError("invalid subkey len")
+    subkey = ctypes.create_string_buffer(subkey_len)
+    si = ctypes.c_uint64(subkey_id)
+    __check(sodium.crypto_kdf_derive_from_key(subkey, ctypes.c_size_t(subkey_len), si, ctx, key))
+    return subkey.raw
+
+# void crypto_kdf_keygen(unsigned char k[crypto_kdf_KEYBYTES])
+def crypto_kdf_keygen():
+    k = ctypes.create_string_buffer(crypto_kdf_KEYBYTES)
+    sodium.crypto_kdf_keygen(k)
+    return k.raw
+
+# int crypto_kdf_hkdf_sha256_extract_init(crypto_kdf_hkdf_sha256_state *state,
+#                                         const unsigned char *salt, size_t salt_len)
+@sodium_version(1, 0, 19)
+def crypto_kdf_hkdf_sha256_extract_init(salt=b''):
+    state = ctypes.create_string_buffer(crypto_kdf_hkdf_sha256_STATEBYTES)
+    __check(sodium.crypto_kdf_hkdf_sha256_extract_init(state, salt, ctypes.c_size_t(len(salt))))
+    return state
+
+# int crypto_kdf_hkdf_sha256_extract_update(crypto_kdf_hkdf_sha256_state *state,
+#                                           const unsigned char *ikm, size_t ikm_len)
+@sodium_version(1, 0, 19)
+def crypto_kdf_hkdf_sha256_extract_update(state, ikm=b''):
+    if len(state) != crypto_kdf_hkdf_sha256_STATEBYTES: raise ValueError("invalid state")
+    __check(sodium.crypto_kdf_hkdf_sha256_extract_update(state, ikm, ctypes.c_size_t(len(ikm))))
+    return state
+
+# int crypto_kdf_hkdf_sha256_extract_final(crypto_kdf_hkdf_sha256_state *state,
+#                                          unsigned char prk[crypto_kdf_hkdf_sha256_KEYBYTES])
+@sodium_version(1, 0, 19)
+def crypto_kdf_hkdf_sha256_extract_final(state):
+    if len(state) != crypto_kdf_hkdf_sha256_STATEBYTES: raise ValueError("invalid state")
+    prk = ctypes.create_string_buffer(crypto_kdf_hkdf_sha256_KEYBYTES)
+    __check(sodium.crypto_kdf_hkdf_sha256_extract_final(state, prk))
+    return prk.raw
+
+# int crypto_kdf_hkdf_sha256_extract(
+#     unsigned char prk[crypto_kdf_hkdf_sha256_KEYBYTES],
+#     const unsigned char *salt, size_t salt_len, const unsigned char *ikm,
+#     size_t ikm_len)
+@sodium_version(1, 0, 19)
+def crypto_kdf_hkdf_sha256_extract(salt=b'', ikm=b''):
+    prk = ctypes.create_string_buffer(crypto_kdf_hkdf_sha256_KEYBYTES)
+    __check(sodium.crypto_kdf_hkdf_sha256_extract(prk, salt, ctypes.c_size_t(len(salt)), ikm, ctypes.c_size_t(len(ikm))))
+    return prk.raw
+
+# void crypto_kdf_hkdf_sha256_keygen(unsigned char prk[crypto_kdf_hkdf_sha256_KEYBYTES])
+@sodium_version(1, 0, 19)
+def crypto_kdf_hkdf_sha256_keygen():
+    k = ctypes.create_string_buffer(crypto_kdf_hkdf_sha256_KEYBYTES)
+    sodium.crypto_kdf_hkdf_sha256_keygen(k)
+    return k.raw
+
+# int crypto_kdf_hkdf_sha256_expand(unsigned char *out, size_t out_len,
+#                                   const char *ctx, size_t ctx_len,
+#                                   const unsigned char prk[crypto_kdf_hkdf_sha256_KEYBYTES])
+@sodium_version(1, 0, 19)
+def crypto_kdf_hkdf_sha256_expand(outlen, prk, ctx=b''):
+    if not (crypto_kdf_hkdf_sha256_BYTES_MIN <= outlen <= crypto_kdf_hkdf_sha256_BYTES_MAX): raise ValueError("invalid output len")
+    if len(prk) != crypto_kdf_hkdf_sha256_KEYBYTES: raise ValueError("invalid prk")
+    out = ctypes.create_string_buffer(outlen)
+    __check(sodium.crypto_kdf_hkdf_sha256_expand(out, ctypes.c_size_t(outlen), ctx, ctypes.c_size_t(len(ctx)), prk))
+    return out.raw
+
+# int crypto_kdf_hkdf_sha512_extract_init(crypto_kdf_hkdf_sha512_state *state,
+#                                         const unsigned char *salt, size_t salt_len)
+@sodium_version(1, 0, 20)
+def crypto_kdf_hkdf_sha512_extract_init(salt=b''):
+    state = ctypes.create_string_buffer(crypto_kdf_hkdf_sha512_STATEBYTES)
+    __check(sodium.crypto_kdf_hkdf_sha512_extract_init(state, salt, ctypes.c_size_t(len(salt))))
+    return state
+
+# int crypto_kdf_hkdf_sha512_extract_update(crypto_kdf_hkdf_sha512_state *state,
+#                                           const unsigned char *ikm, size_t ikm_len)
+@sodium_version(1, 0, 20)
+def crypto_kdf_hkdf_sha512_extract_update(state, ikm=b''):
+    if len(state) != crypto_kdf_hkdf_sha512_STATEBYTES: raise ValueError("invalid state")
+    __check(sodium.crypto_kdf_hkdf_sha512_extract_update(state, ikm, ctypes.c_size_t(len(ikm))))
+    return state
+
+# int crypto_kdf_hkdf_sha512_extract_final(crypto_kdf_hkdf_sha512_state *state,
+#                                          unsigned char prk[crypto_kdf_hkdf_sha512_KEYBYTES])
+@sodium_version(1, 0, 20)
+def crypto_kdf_hkdf_sha512_extract_final(state):
+    if len(state) != crypto_kdf_hkdf_sha512_STATEBYTES: raise ValueError("invalid state")
+    prk = ctypes.create_string_buffer(crypto_kdf_hkdf_sha512_KEYBYTES)
+    __check(sodium.crypto_kdf_hkdf_sha512_extract_final(state, prk))
+    return prk.raw
+
+# int crypto_kdf_hkdf_sha512_extract(
+#     unsigned char prk[crypto_kdf_hkdf_sha512_KEYBYTES],
+#     const unsigned char *salt, size_t salt_len, const unsigned char *ikm,
+#     size_t ikm_len)
+@sodium_version(1, 0, 19)
+def crypto_kdf_hkdf_sha512_extract(salt=b'', ikm=b''):
+    prk = ctypes.create_string_buffer(crypto_kdf_hkdf_sha512_KEYBYTES)
+    __check(sodium.crypto_kdf_hkdf_sha512_extract(prk, salt, ctypes.c_size_t(len(salt)), ikm, ctypes.c_size_t(len(ikm))))
+    return prk.raw
+
+# void crypto_kdf_hkdf_sha512_keygen(unsigned char prk[crypto_kdf_hkdf_sha512_KEYBYTES])
+@sodium_version(1, 0, 19)
+def crypto_kdf_hkdf_sha512_keygen():
+    k = ctypes.create_string_buffer(crypto_kdf_hkdf_sha512_KEYBYTES)
+    sodium.crypto_kdf_hkdf_sha512_keygen(k)
+    return k.raw
+
+# int crypto_kdf_hkdf_sha512_expand(unsigned char *out, size_t out_len,
+#                                   const char *ctx, size_t ctx_len,
+#                                   const unsigned char prk[crypto_kdf_hkdf_sha512_KEYBYTES])
+@sodium_version(1, 0, 19)
+def crypto_kdf_hkdf_sha512_expand(outlen, prk, ctx=b''):
+    if not (crypto_kdf_hkdf_sha512_BYTES_MIN <= outlen <= crypto_kdf_hkdf_sha512_BYTES_MAX): raise ValueError("invalid output len")
+    if len(prk) != crypto_kdf_hkdf_sha512_KEYBYTES: raise ValueError("invalid prk")
+    out = ctypes.create_string_buffer(outlen)
+    __check(sodium.crypto_kdf_hkdf_sha512_expand(out, ctypes.c_size_t(outlen), ctx, ctypes.c_size_t(len(ctx)), prk))
+    return out.raw
+
 # int crypto_kx_keypair(unsigned char pk[crypto_kx_PUBLICKEYBYTES],
 #                      unsigned char sk[crypto_kx_SECRETKEYBYTES]);
 @sodium_version(1, 0, 12)
